@@ -1,8 +1,8 @@
-## ace.R (2014-05-27)
+## ace.R (2016-06-08)
 
 ##   Ancestral Character Estimation
 
-## Copyright 2005-2014 Emmanuel Paradis and Ben Bolker
+## Copyright 2005-2016 Emmanuel Paradis and Ben Bolker
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -227,10 +227,12 @@ ace <-
                 if (is.na(dev)) Inf else dev
             }
         } else {
-            E <- if (use.expm) {
-                library(expm)
-                get("expm", "package:expm") # to avoid Matrix::expm
-            } else matexpo
+            if (!requireNamespace("expm", quietly = TRUE) && use.expm) {
+                warning("package 'expm' not available; using function 'matexpo' from 'ape'")
+                use.expm <- FALSE
+            }
+            E <- if (use.expm) expm::expm # to avoid Matrix::expm
+                 else matexpo
             dev <- function(p, output.liks = FALSE) {
                 if (any(is.nan(p)) || any(is.infinite(p))) return(1e50)
                 comp <- numeric(nb.tip + nb.node) # from Rich FitzJohn
@@ -317,7 +319,7 @@ AIC.ace <- function(object, ..., k = 2)
 anova.ace <- function(object, ...)
 {
     X <- c(list(object), list(...))
-    df <- sapply(lapply(X, "[[", "rates"), length)
+    df <- lengths(lapply(X, "[[", "rates"))
     ll <- sapply(X, "[[", "loglik")
     ## check if models are in correct order
     dev <- c(NA, 2*diff(ll))
